class AuthSystem < Sinatra::Base
  helpers Sinatra::Cookies
  enable :sessions

  def user_id_decoded cookies
    begin
      decoded = JWT.decode(cookies["auth"], LifePex::SECRET)
      decoded[0]["user_id"]
    rescue => err
      STDERR.puts "user_id_decoded: #{err}"
      nil
    end
  end

  def logged_in?
    if cookies["auth"]
      user_id_decoded cookies
    else
      nil
    end
  end

  def current_user
    if @current_user
      @current_user
    elsif logged_in?
      @current_user = LifePex::User.where(id: user_id_decoded(cookies)).first
    else
      raise RuntimeError.new "Not connected"
    end
  end

  set(:auth) do
    condition do
      unless logged_in?
        redirect "/login", 303
      end
    end
  end
end
