require "yaml"

class UserSystem < AuthSystem
  use AuthSystem

  DEFAULT_PEXS_FOR_NEW_USERS = YAML.load_file "config/default_pexs_for_new_users.yaml"

  get "/login" do
    slim :login
  end

  post "/login" do
    user = LifePex::User.where(username: params["username"]).first
    if user && BCrypt::Password.new(user[:hashed_password]) == params["password"]
      cookies["auth"] = JWT.encode({ "user_id" => user[:id] }, LifePex::SECRET)
      redirect "/"
    else
      slim :login, locals: { flash: { danger: "Failed to login" } }
    end
  end

  get "/register" do
    slim :register
  end

  post "/register" do
    user = LifePex::User.where(username: params["username"]).first
    if !user
      hashed_password = BCrypt::Password.create params["password"]
      user_id = LifePex::User.insert(username: params["username"], hashed_password: hashed_password)
      cookies["auth"] = JWT.encode({ "user_id" => user_id }, LifePex::SECRET)
      DEFAULT_PEXS_FOR_NEW_USERS.each do |pex|
        LifePex::Pex.insert(**pex, user_id: user_id)
      end
      redirect "/"
    else
      slim :register, locals: { flash: { danger: "Failed to register as #{params['username']}" } }
    end
  end

  get "/logout", auth: [] do
    cookies.delete "auth"
    slim :logout, locals: { flash: { success: "Logged out" } }
  end

  get "/password", auth: [] do
    slim :password
  end

  post "/password", auth: [] do
    if params["password"] != params["password_confirmation"]
      slim :password, locals: { flash: { warning: "Password don't match" } }
    else
      hashed_password = BCrypt::Password.create params["password"]
      current_user.update(hashed_password: hashed_password)
      slim :password, locals: { flash: { success: "Password updated" } }
    end
  end
end
